/*
 * Copyright 2013 Linear Technology Corp. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * ltc3676_test.c
 *
 * User test application to verify driver functions of LTC3676-1
 *
 * Created on: Jul 5, 2013
 *      Author: novtech
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h> 

void  doMenu();
void  menu_app();
void  displayMenu();
char  getSelection();
void  processChoice(const char *userChoice);
void  get_dcdc_enable(int name);
void  get_dcdc_mode(int name);
void  get_dcdc_voltage(int name);
void  set_dcdc_voltage(int name, int voltage);
int   get_regulator_name(char *name);

char user_choice;

#define SW3 		 "vddarm_ext"
#define SW2 		 "vddsoc_ext"
#define NAME_CMD    	 "head /sys/class/regulator/regulator.%d/name"
#define STATE_CMD    	 "head /sys/class/regulator/regulator.%d/state"
#define MODE_CMD  	 "head /sys/class/regulator/regulator.%d/opmode"
#define GET_VOLTAGE_CMD  "head /sys/class/regulator/regulator.%d/microvolts"
#define SET_VOLTAGE_CMD  "echo -n %d > /sys/class/regulator/regulator.%d/microvolts"
#define NUM_LTC_DEV 	  2

int main(int argc, char **argv)
{
	printf("Novtech LTC3676 Test Application!\n");
	
	menu_app();

	return 0;
}

void menu_app()
{
	while (1)
	{
		sleep(1);
		doMenu();
	}
}

void doMenu()
{
  if (user_choice != 0x20 )
	  displayMenu();
  user_choice = getSelection();
  processChoice(&user_choice);

}

void displayMenu()
{
  printf("\n");
  printf("1. VDDARM_EXT Enable Status\n");
  printf("2. VDDSOC_EXT Enable Status\n");
  printf("3. VDDARM_EXT Get Mode Status\n");
  printf("4. VDDSOC_EXT Get Mode Status\n");
  printf("5. VDDARM_EXT Get Voltage\n");
  printf("6. VDDSOC_EXT Get Voltage\n");
  printf("7. VDDARM_EXT Set Voltage 1378388\n");
  printf("8. VDDARM_EXT Set Voltage 1402570\n");
  printf("9. VDDARM_EXT Set Voltage 1426752\n");
  printf("x. Quit\n");
  printf("Please enter your choice: ");
}

char getSelection()
{
   char in;

   in = getchar();

   if (in != '\n')
	   return in;
   else
	   return ' ';
}

int get_regulator_name(char *name)
{
    FILE *fpipe;
    char command[100];
    char line[11];
    int i = 0;
    DIR           *d;
    struct dirent *dir;
 
    d = opendir("/sys/class/regulator/");

    if (d)
    {
    	while ((dir = readdir(d)) != NULL)
    	{
      		//printf("Dir: %s\n", dir->d_name);
		if ((strcmp(dir->d_name, ".") == 0) || (strcmp(dir->d_name, "..") == 0))
			continue;
		else
		{
			char tmp[11];
			int len;

			sprintf(command, NAME_CMD, i);
			if (0 == (fpipe = (FILE*)popen(command, "r")))
			{
				perror("popen() failed.");
				break;
			}
			fgets(line, sizeof(line), fpipe);
			strcpy(tmp, line);	
			if (strcmp(tmp, name) == 0)
			{
				pclose(fpipe);
				break;
			}
			i++;
		}
    	}
    	closedir(d);
    }
    return i;
}

void processChoice(const char *userChoice)
{
	int name;

	if(*userChoice == '1')
	{
	    name = get_regulator_name(SW3);
	    system("echo LTC3676 vddarm_ext Enable: ");
	    get_dcdc_enable(name);
	}
	if(*userChoice == '2')
	{
	    name = get_regulator_name(SW2);
	    system("echo LTC3676 vddsoc_ext Enable: ");
	    get_dcdc_enable(name);
	}
	else if(*userChoice == '3')
	{
	      name = get_regulator_name(SW3);
	      system("echo LTC3676 vddarm_ext Mode: ");
	      get_dcdc_mode(name);
	}
	else if(*userChoice == '4')
	{
	      name = get_regulator_name(SW2);
      	      system("echo LTC3676 vddsoc_ext Mode: ");
	      get_dcdc_mode(name);
	}
	else if(*userChoice == '5')
	{
	      name = get_regulator_name(SW3);
	      system("echo LTC3676 vddarm_ext Get Voltage: ");
	      get_dcdc_voltage(name);
	}
	else if(*userChoice == '6')
	{
	      name = get_regulator_name(SW2);
	      system("echo LTC3676 vddsoc_ext Get Voltage: ");
	      get_dcdc_voltage(name);
	}
	else if(*userChoice == '7')
	{
	      name = get_regulator_name(SW3);
	      system("echo LTC3676 vddarm_ext Set Voltage 1378388: ");
	      set_dcdc_voltage(name, 1378388);
	}
	else if(*userChoice == '8')
	{
	      name = get_regulator_name(SW3);
	      system("echo LTC3676 vddarm_ext Set Voltage 1402570: ");
	      set_dcdc_voltage(name, 1402570);
	}
	else if(*userChoice == '9')
	{
	      name = get_regulator_name(SW3);
	      system("echo LTC3676 vddarm_ext Set Voltage 1426752: ");
	      set_dcdc_voltage(name, 1426752);
	}
	else if(*userChoice == 'x')
	{
	   exit(-1);
	}
	else if(*userChoice == ' ')
	{
	   return;
	}
	else
   	   printf("Please Enter a valid choice:\n");
}

void get_dcdc_enable(int name)
{
	char command[150];
	sleep(1);
	sprintf(command, STATE_CMD, name);
	system(command);
}

void get_dcdc_mode(int name)
{
	char command[150];
	sleep(1);
	sprintf(command, MODE_CMD, name);
	system(command);
}

void get_dcdc_voltage(int name)
{
	char command[150];
	sprintf(command, GET_VOLTAGE_CMD, name);
	system(command);
}

void set_dcdc_voltage(int name, int voltage)
{
	char command[256];
	sleep(1);
	sprintf(command, SET_VOLTAGE_CMD, voltage, name);
	system(command);
	sleep(1);
	//Get Voltage to verify
	get_dcdc_voltage(name);
}

